/*
 *
 *		UC16In.cpp
 *		
 *		part of the Evolution UC-16 driver for BeOS R5
 *
 *		stefan@keindesign.de	http://www.keindesign.de/stefan/
 *
 *		              Copyright (c) 2002 Stefan Werner
 *					
 *		Permission is hereby granted, free of charge, to any person 
 *		obtaining a copy of this software and associated documentation files 
 *		(the "Software"), to deal in the Software without restriction, 
 *		including without limitation the rights to use, copy, modify, merge, 
 *		publish, distribute, sublicense, and/or sell copies of the Software, 
 *		and to permit persons to whom the Software is furnished to do so, 
 *		subject to the following conditions:
 *		The above copyright notice and this permission notice shall be 
 *		included in all copies or substantial portions of the Software.
 *
 *		THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 *		EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 *		MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
 *		NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS 
 *		BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN 
 *		ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
 *		CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 *		SOFTWARE
 *
 */

#include "UC16In.hpp"
#include "USBMIDI.h"
#include <stdio.h>
#include <malloc.h>

UC16In::UC16In(const USBEndpoint *point, const char* name) {
	uc16 = point->Device();
	ept = point;

	m_producer = new BMidiLocalProducer (name);
	if (m_producer) {
		m_producer->Register();
		running = true;
		t = spawn_thread(UC16In::ThreadEntry, name, B_REAL_TIME_PRIORITY,this);
		resume_thread(t);
	}
}

UC16In::~UC16In(){
	if (m_producer);
	m_producer->Release();
	m_producer = NULL;
	if(t >= 0){
		status_t res;
		running = false;
		wait_for_thread(t,&res);
		t = -1;
	}
}

status_t UC16In::Thread(void){
	status_t err;
	int i, j = 0;
	err = uc16->ControlTransfer (USB_REQTYPE_STANDARD | USB_REQTYPE_ENDPOINT_OUT, USB_REQUEST_CLEAR_FEATURE, USB_FEATURE_ENDPOINT_HALT, 1, 0, NULL);
	if (err != B_OK)
		fprintf (stderr, "control request failed\n");
	while (running) {
		i++;
		err = ept->BulkTransfer(data, 64);
		if (err == B_OK) {
			// here one would distinguish different Midi in ports of the interface
			// and the various Midi messages they can send. As my UC-16 has no real
			// Midi in but just 16 dials that send Control Change messages, I can't
			// test any of the other messages right now.
			switch (data[0]) {
				case 11: {
					j++;
					uchar channel = data[1] & 0x0f;
					uchar controlNumber = data[2];
					uchar controlValue = data[3];
					if (m_producer) {
						m_producer->SprayControlChange(channel, controlNumber, controlValue, 0);
					}
					break;
				}
			}
		}
	}
	fprintf (stderr, "sended %d MIDI Events\n", j);
	return B_OK;
}
