/*
 *
 *		uc16.cpp
 *		
 *		part of the Evolution UC-16 driver for BeOS R5
 *
 *		stefan@keindesign.de	http://www.keindesign.de/stefan/
 *
 *		              Copyright (c) 2002 Stefan Werner
 *					
 *		Permission is hereby granted, free of charge, to any person 
 *		obtaining a copy of this software and associated documentation files 
 *		(the "Software"), to deal in the Software without restriction, 
 *		including without limitation the rights to use, copy, modify, merge, 
 *		publish, distribute, sublicense, and/or sell copies of the Software, 
 *		and to permit persons to whom the Software is furnished to do so, 
 *		subject to the following conditions:
 *		The above copyright notice and this permission notice shall be 
 *		included in all copies or substantial portions of the Software.
 *
 *		THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 *		EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 *		MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
 *		NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS 
 *		BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN 
 *		ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
 *		CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 *		SOFTWARE
 *
 */

#include <stdio.h>
#include <Directory.h>

#include "USBKit.h"

#include "UC16Out.hpp"
#include "UC16In.hpp"

class UC16Roster : public USBRoster
{
public:
	UC16Roster ();
	virtual status_t DeviceAdded(USBDevice *dev);
	virtual void DeviceRemoved(USBDevice *dev);
private:
	BList outPorts;
	BList inPorts;
};

UC16Roster::UC16Roster() {
}

status_t  UC16Roster::DeviceAdded(USBDevice *dev) {
	const USBConfiguration *config;
	const USBInterface *iface;
	const USBEndpoint *ept;

	unsigned int i,j,k;

	char endpointName [256];

	if (!dev)
		return B_ERROR;
		
	if (dev->InitCheck() != B_OK)
		fprintf (stderr, "dev->InitCheck() failed!\n");
	
//	if ((dev->VendorID() == 0x0a4d) && (dev->ProductID() == 0x00f0) && dev->Version() >= 0x0124)
		for (k = 0; k < dev->CountConfigurations(); k++)
			if ((config = dev->ConfigurationAt(k)))
				for (j = 0 ; j < config->CountInterfaces(); j++)
					if ((iface = config->InterfaceAt(j)))
						if ((iface->Class() == 1) && (iface->Protocol() == 0) && (iface->Subclass() == 3)) {
							printf ("Found valid device: \n");
							const char* prodID = dev->ProductString();
							printf ("%s\n%s\n",dev->ManufacturerString(), prodID);
							dev->SetConfiguration(dev->ConfigurationAt(0));
							for(i = 0; i < iface->CountEndpoints(); i++) {
								if((ept = iface->EndpointAt(i)) && (ept->IsOutput())) {
									int numItems = outPorts.CountItems() + 1;
									sprintf (endpointName, "%s Out #%d", prodID, numItems);
									printf ("Creating MidiConsumer: %s\n",endpointName);
									outPorts.AddItem(new UC16Out(ept, endpointName));
								}
								else if ((ept = iface->EndpointAt(i)) && (ept->IsInput())) {
									int numItems = inPorts.CountItems() + 1;
									sprintf (endpointName, "%s In #%d", prodID, numItems);
									printf ("Creating MidiProducer: %s\n",endpointName);
									inPorts.AddItem(new UC16In (ept, endpointName));
								}
							}
						}
	return B_OK;
}

void UC16Roster::DeviceRemoved(USBDevice *dev) {
	int i;
	UC16In *in;
	UC16Out *out;
	
	for (i = 0; (in = (UC16In*)inPorts.ItemAt(i)); i++){
		if(in->GetDevice() == dev){
			printf ("Removing MidiProducer\n");
			inPorts.RemoveItem(i);
			delete in;
		}
	}
	for (i = 0; (out = (UC16Out*)outPorts.ItemAt(i)); i++){
		if(out->GetDevice() == dev){
			printf ("Removing MidiConsumer\n");
			outPorts.RemoveItem(i);
			out->Release();
			//delete out;
		}
	}
	return;
}

int main(int argc, char *argv[])
{
	printf ("UC-16 driver (c) 2002 Stefan Werner \n");
	(new BDirectory("/dev/bus/usb/"))->CountEntries();
	UC16Roster *roster = new UC16Roster();
	roster->Start();
	while(1) {
		// wait around and let the roster do its thing
		snooze(1000000); 
	}
	
	return 0;
}

